#pragma section=".ee_mem"
#define EE_END (u32)__section_end(".ee_mem")

const int PG_LN = 0x400; //   
const int PGM_NUM = 20;  //   
//      c  
const int PGM_SIZE = (PG_LN / PGM_NUM / 4)*4 - 4;

//    
struct EE_store
{
 u8 pg_step[PGM_SIZE];
 u32 crc;
};

__root __no_init const EE_store See[PGM_NUM] @ ".ee_mem";

const int flash_prog_size = sizeof(See[0].pg_step);

//     
bool check_pgm_space(const int pgm_num)
{
 u32 *pt = (u32*)&See[pgm_num];

 for(int i=0; i < sizeof(EE_store)/4; i++)
  if(*(pt+i) != (u32)-1) return false;

 return true;
}

//    (   nop,     "See")
void write_flash_pgm(u8 input[], const int pgm_num)
{
 u32 *pt;
 u32 crc;

 // crc    
 crc = calc_pgm_crc(input, flash_prog_size);

 //  flash    
 if(check_pgm_space(pgm_num)==false)
 {
  static u32 buff[PG_LN/4];

  // 1kb   
  for(int i = 0; i < PG_LN/4; i++) buff[i] = *((u32*)See + i);

  //      
  u32 offset = (u32*)(&See[pgm_num]) - (u32*)See;

  //      "buff"  
  pt = buff + offset;

  //     "buff"
  int i = 0;
  for(; i<PGM_SIZE/4; i++) *(pt+i) = *((u32*)input + i);

  // crc     "buff"
  *(pt+i) = crc;

  __disable_interrupt();
  FLASH_Unlock();

  //  flash   "See"
  while(FLASH_ErasePage((uint32_t)See)==FLASH_BUSY) continue;

  // 1kb  "buff"   flash     "See"
  for(int i = 0; i < PG_LN/4; i++)
  {
   uint32_t addr = (u32)((u32*)See + i);
   u32      data = buff[i];

   while(FLASH_ProgramWord(addr, data)==FLASH_BUSY) continue;
  }
 }
 //   
 else
 {
  __disable_interrupt();
  FLASH_Unlock();

  pt = (u32*)&See[pgm_num];

  //    flash
  int i = 0;
  for(; i < PGM_SIZE/4; i++)
  {
   uint32_t addr = (u32)(pt + i);
   u32      data = *((u32*)input+i);

   while(FLASH_ProgramWord(addr, data)==FLASH_BUSY) continue;
  }

  // crc
  uint32_t addr = (u32)(pt + i);
  u32      data = crc;

  while(FLASH_ProgramWord(addr, data)==FLASH_BUSY) continue;
 }

  FLASH_Lock();
 __enable_interrupt();
}

int get_program_length(void)
{
 for(int i = sizeof(ee_buff)-1; i; i--)
  if(ee_buff[i] != (u8)OP_NULL) return i+1;

 return 0;
}
