/********************************************************
Workspace: IAR_ARM+SPL
MCU      : STM32f030C6T6
Project  : RPN calculator
Version  : 3h
Date     : NOV 2020
*********************************************************/
#include <stm32f0xx.h>

#include <stdbool.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <stm32f0xx_gpio.h>
#include <stm32f0xx_pwr.c>
#include <stm32f0xx_exti.c>
#include <stm32f0xx_misc.c>
#include <stm32f0xx_syscfg.c>
#include <stm32f0xx_rcc.c>
#include <stm32f0xx_flash.c>

#include "bitset.c"
#include "defines.c"
#include "periph_cfg.c"
#include "kb.c"
#include "beep_works.c"
#include "GLCD.c"
#include "display_work.c"
#include "ee_i2c.c"
#include "storage_routines.c"
#include "opstrings.c"
#include "bat_control.c"
#include "led.c"
#include "calculator_routines.c"
#include "interrupts.c"
#include "crc_calculation.c"
#include "eep_em.c"

bool devcon(u8*);
u8 tmp_key;

void main()
{
 cfg.rcc();
 cfg.gpio();

 bat_charge(true);

 cfg.systic(1000);
 cfg.t17(1000);

 DISP_POW(1);

 cfg.I2C();
 rst_I2C();

 tim2 = TOFF_CNTDN;  // 

 kb_init();
 adc_config();
 lcd.Init();


 while(1)
 {
  static u8 keys[KEY_DEPTH]={B_NULL,B_NULL,B_NULL};
  static bool kc_trg=0;
  static bool devcon_trg = 1;
  static u32 cntdn;           // 
  static bool off_trg;
  static bool kb_led_trg;     //     
  static bool wakeup_skip_key_trg;
  static char ts_0[17], ts_1[17];


  bool lcd_ret = lcd.send_screen(s0,s1);

  tmp_key = ret_key;

  //  
  if(tmp_key == 0xff)
  {
   kc_trg = 0;
   if(!devcon_trg)__WFI();

   //   
   if(off_trg == true)
   {
    off_trg = false;
    goto m_off;
   }
  }
  else if(!kc_trg) //   
  {
   if(keys[0] != tmp_key || cntdn == 0)
   {
    kc_trg = 1;
    beep(40);
    cntdn = 150;

    kb_led_trg = true;

    tim2 = TOFF_CNTDN; //     

    //  
    for(u8 i=KEY_DEPTH-1;i;i--) keys[i]=keys[i-1];
    keys[0] = tmp_key;

    if(keys[0] == 0 && keys[1] == 0 && keys[2] == 0)
    {
     off_trg = true;
     keys[0] = keys[1] = keys[2] = 0xff;
    }

    if(wakeup_skip_key_trg == true)
    {
     wakeup_skip_key_trg = false;

     strcpy(s0,ts_0);
     strcpy(s1,ts_1);

     ops = OP_NULL;
     devcon_trg=1;
    }
    else key2ops_processing(keys, ops, num_mode);


    if(ops!=OP_NULL) devcon_trg=1;
   }
  }

  if(cntdn) cntdn--;

  if(lcd_ret) if(devcon_trg) devcon_trg = devcon(keys);


  //  -
  //  /   
  {
   static int bl_cntdn;

   //    
   if(kb_led_trg == true)
   {
    kb_led_trg = false;

    //   
    bl_cntdn = 0;
    //      
    backlight(backlight_switch);
   }
   else
   {
    //      
    if(bl_cntdn < 10*1000) bl_cntdn++;
    // 
    else
    {
     bl_cntdn = 0;
     backlight(false);
    }
   }
  }

  //  ,   
  //   
  if(!tim1)
  {
   if(do_conversion(&Vbat)==true)
   {
    //  
    static bool chg_dev_trg;
    //        
    static bool bd_trg;

    tim1 = 1000;

    if(Vbat > 4200)
    {
     bat_charge(false);
     chg_dev_trg = true;

     if(bd_trg == false)
     {
      bd_trg = true;
      snprintf(s0,sizeof(s0),"Vbat: READY     ");
     }
    }
    else if(Vbat < 4200-400)
    {
     bd_trg = false;

     bat_charge(true);

     //   
     chg_dev_trg = chg_check(Vbat);
    }

    //    
    if(Vbat < 3300)
    {
     int cnt = 0;

     cnt++;

     if(cnt > 59)
     {
      cnt = 0;

      snprintf(s0,sizeof(s0),"Low Vbat: %u%c%02u%c",Vbat/1000,'.',Vbat%1000/10,'V');

      backlight_switch = false;
      backlight(backlight_switch);
     }
    }

    //        
    if(chg_dev_trg == true) tim2 = TOFF_CNTDN;
    //  
    else if(!tim2)
    {
     m_off:

     //   
     tim2 = 500;
     while(tim2)continue;

     //  
     RS      (0);
     E       (0);
     OD7     (0);
     OD6     (0);
     OD5     (0);
     OD4     (0);
     DISP_POW(0);

     __disable_interrupt();
     //    
     kb_init_es();
     // 
     led(false);
     backlight(false);
     //   
     RCC->APB1ENR |= RCC_APB1ENR_PWREN;
     //       
     bat_charge(false);
     //    
     cfg.exti();

     // 
     PWR_EnterSTOPMode(PWR_Regulator_LowPower, PWR_STOPEntry_WFE);

/*------------------------------------------------------------------*/

     //
     bat_charge(true);
     DISP_POW(1);
     cfg.kb_exti_deinit();
     __enable_interrupt();
     lcd.Init();
     RCC->APB1ENR &= ~RCC_APB1ENR_PWREN;

     strcpy(ts_0,s0);
     strcpy(ts_1,s1);

     snprintf(s0,sizeof(s0),"WAKEUP COMPLETE ");
     snprintf(s1,sizeof(s1),"press any key.. ");

     tim2 = TOFF_CNTDN;
     backlight(backlight_switch);
     wakeup_skip_key_trg = true;
    }
   }
  }
 }
}


bool devcon(u8 keys[])
{
 static bool itr=0;
 static enum
 {
  start, work, ee_prog_run, flash_prog_run, prog_edit
 } dev_st = start;

 static u8 ee_bnum;
 u8 retval = 0;

 m_mcycle:

 switch (dev_st)
 {
  case start:
  {
   if(!itr)
   {
    itr=1;

    //    eeprom    
    //        
    ee_bnum = check_eeprom_bnum();

    //     eeprom   
    //    
    if(ee_bnum) restore_options_from_ram(&options);
    else        options.precision = DEF_PRECISION;

    if(get_crc()==false) {strcpy(s0,"FLASH CRC ERROR!");tim0 = 1000;}
    else if(ee_bnum ==1) {strcpy(s0,"EEPROM: 24xx02  ");tim0 = 200; }
    else if(ee_bnum ==2) {strcpy(s0,"EEPROM: 24xx04  ");tim0 = 200; }
    else if(ee_bnum ==4) {strcpy(s0,"EEPROM: 24xx08  ");tim0 = 200; }
    else ee_bnum = 0;

    for(u8 i=0;i<sizeof(ee_buff);i++)ee_buff[i]=OP_NULL;
    beep(30);
   }

   if(!tim0) dev_st = work;

   retval = 1;

   if(dev_st!=start){itr=0;}
  }
  break;

  case work:
  {
   if(!itr)
   {
    itr=1;
    for(u8 i=0;i<KEY_DEPTH;i++)keys[i]=B_NULL;
    ops = OP_NULL;

    goto m_mcycle;
   }
   else
   {
    static bool prec_trg=0;
    //      
    if(ops == OP_PREC || prec_trg)
    {
     if(ops == OP_PREC)
     {
      prec_trg=1;
      dop_key_trg = 1;

      if(options.precision > 0x0F)options.precision = 0x0F;

      snprintf(s0,sizeof(s0),"SET PRECISION(%X%c",options.precision,')');
      strcpy(s1,"IN RANGE 0..F:  ");
     }
     else
     {
      switch(ops)
      {
       case OP_0:
       case OP_1:
       case OP_2:
       case OP_3:
       case OP_4:
       case OP_5:
       case OP_6:
       case OP_7:
       case OP_8:
       case OP_9:
       case OP_A:
       case OP_B:
       case OP_C:
       case OP_D:
       case OP_E:
       case OP_F:
       {
        prec_trg = 0;
        dop_key_trg = 0;
        u8 tmp_precision = options.precision;

        switch(ops)
        {
         case OP_0: tmp_precision = 0; break;
         case OP_1: tmp_precision = 1; break;
         case OP_2: tmp_precision = 2; break;
         case OP_3: tmp_precision = 3; break;
         case OP_4: tmp_precision = 4; break;
         case OP_5: tmp_precision = 5; break;
         case OP_6: tmp_precision = 6; break;
         case OP_7: tmp_precision = 7; break;
         case OP_8: tmp_precision = 8; break;
         case OP_9: tmp_precision = 9; break;
         case OP_A: tmp_precision =10; break;
         case OP_B: tmp_precision =11; break;
         case OP_C: tmp_precision =12; break;
         case OP_D: tmp_precision =13; break;
         case OP_E: tmp_precision =14; break;
         case OP_F: tmp_precision =15; break;
        }

        if(options.precision != tmp_precision)
        {
         options.precision = tmp_precision;
         save_options(&options);
        }

        disp_output_processing(ops, num_mode);
       }
      }
     }
    }
    else
    {
     //    ,   
     bool ssc_trg = num_mode_selector(ops, num_mode);

     //   
     if(ops == OP_BX)
     {
      //restore stack
      for(int i=0;i<PIT_DEPTH;i++) pit[i] = shadow_pit[i];
     }
     else
     {
      //copy stack to shadow
      for(int i=0;i<PIT_DEPTH;i++)shadow_pit[i] = pit[i];
     }

     //   
     opc_mode_selector(ops, num_mode, opc_mode);
     //      
     opc_mode_change_processing(ops, num_mode, opc_mode);

     //       
     switch(opc_mode)
     {
      //   
      case opc_processing:
      {
       opcode_processing(ops, prev_ops, ee_bnum);
       disp_output_processing(ops, num_mode);
      }break;
      //   
      case edit_mnt      : edit_mnt_processing(ops, num_mode);break;
      //   
      case edit_exp      : edit_exp_processing(ops);          break;
     }

     prev_ops = ops;

     //     ,   
     if(ssc_trg)
     {
      switch(num_mode)
      {
       case dec: strcpy(s0,"             DEC");break;
       case hex: strcpy(s0,"             HEX");break;
       case bin: strcpy(s0,"             BIN");break;
      }
     }

     if     (ops == OP_PGMM)  {dev_st = prog_edit; retval = 1;}
     else if(ops == OP_FPRUN) {dev_st = ee_prog_run; retval = 1;}
     else if(ops == OP_KPRUN) {dev_st = flash_prog_run; retval = 1;}
     else if(ops == OP_RPRUN)
     {
      prg_r(ee_buff,1,0);
      while(prg_r(ee_buff) < sizeof(ee_buff)) continue;
      ops = OP_NULL;
      retval = 1;
     }
     else if(ops == OP_VBAT)  {snprintf(s0,sizeof(s0),"    Vbat: %u%c%02u%c",Vbat/1000,'.',Vbat%1000/10,'V');}
     else if(ops == OP_RDG)
     {
      if(angle_mode == ang_deg)
      {
       angle_mode = ang_rad;
       strcpy(s0,"             RAD");
      }
      else
      {
       angle_mode = ang_deg;
       strcpy(s0,"             DGR");
      }
     }
    }
   }

   if(dev_st!=work){itr=0;}
  }
  break;

  case ee_prog_run:
  {
   if(!itr)
   {
    bool trg = false;

    //  eeprom   ,   
    if(ee_bnum < 2)
    {
     strcpy(s0,"NOT AVAILABLE   ");
     strcpy(s1,"press any key.. ");

     dev_st = work;
    }
    //   
    else if(ee_bnum < 3)
    {
     bank = 1;
     trg = true;
    }
    //   eeprom
    else
    {
     strcpy(s0,"EEPROM BANK NUM?");
     strcpy(s1,"(1 2 3)available");

     //   
     if     (ops == OP_1) {bank = 1; trg = true;}
     else if(ops == OP_2) {bank = 2; trg = true;}
     else if(ops == OP_3) {bank = 3; trg = true;}
    }

    if(trg == true)
    {
     itr = 1;

     // 
     i2c_bank_read(bank, ee_buff);
     beep(300);

     //
     prg_r(ee_buff, 1);
     goto m_mcycle;
    }
   }
   //   
   else
   {
    while(prg_r(ee_buff) < sizeof(ee_buff)-1) continue;

    dev_st = work;
    itr = 0;
    retval = 1;
   }
  }
  break;

  case flash_prog_run:
  {
   //   
   if(!itr)
   {
    bool trg = false;

    strcpy(s0,"FLASH PROG NUM? ");
    strcpy(s1,"(0...%)available");

    //     
    switch(ops)
    {
     case OP_0   : bank = 0 ; trg = true; break;
     case OP_COM : bank = 1 ; trg = true; break;
     case OP_MNS : bank = 2 ; trg = true; break;
     case OP_SEXP: bank = 3 ; trg = true; break;
     case OP_Cx  : bank = 4 ; trg = true; break;
     case OP_1   : bank = 5 ; trg = true; break;
     case OP_2   : bank = 6 ; trg = true; break;
     case OP_3   : bank = 7 ; trg = true; break;
     case OP_XY  : bank = 8 ; trg = true; break;
     case OP_ENT : bank = 9 ; trg = true; break;
     case OP_4   : bank = 10; trg = true; break;
     case OP_5   : bank = 11; trg = true; break;
     case OP_6   : bank = 12; trg = true; break;
     case OP_SUM : bank = 13; trg = true; break;
     case OP_MUL : bank = 14; trg = true; break;
     case OP_7   : bank = 15; trg = true; break;
     case OP_8   : bank = 16; trg = true; break;
     case OP_9   : bank = 17; trg = true; break;
     case OP_SBT : bank = 18; trg = true; break;
     case OP_DIV : bank = 19; trg = true; break;
    }

    //  
    if(trg == true)
    {
     itr = 1;

     //  
     init_prog_memory(ee_buff);
     //    
     for(int i=0; i < flash_prog_size; i++) ee_buff[i] = See[bank].pg_step[i];
     //
     prg_r(ee_buff, 1);

     // crc
     u32 crc = calc_pgm_crc((u8*)See[bank].pg_step, flash_prog_size);

     //crc  -  
     if(crc == See[bank].crc)
     {
      beep(300);

      goto m_mcycle;
     }
     //crc  ,   (    F,Cx)
     else
     {
      strcpy(s0,"PROG CRC FAILED ");
      strcpy(s1,"press any key.. ");
     }
    }
   }
   // 
   else
   {
    while(prg_r(ee_buff) < flash_prog_size) continue;

    dev_st = work;

    itr = 0;
    retval = 1;
   }
  }
  break;

  case prog_edit:
  {
   static enum {load, edit, save}edit_stage;
   static Storage_source source;
   static bool pc_reset;
   static int prog_cnt;
   static bool miss_trg;

   if(!itr)
   {
    strcpy(s0,"EDIT PROG MODE  ");
    strcpy(s1,"LOAD? YES:1 NO:0");

    pc_reset = 1;

    //     ,   
    if (ops == OP_0)
    {
     itr=1;

     edit_stage = edit;

     ops = OP_NULL;

     goto m_mcycle;
    }
    //   
    else if(ops == OP_1)
    {
     itr = 1;
     edit_stage  = load;
     //    
     prog_choice(source, bank, ops, ee_bnum, 1);
    }
   }
   else
   {
    switch (edit_stage)
    {
     //    
     case load:
     {
      if(prog_choice(source, bank, ops, ee_bnum)==true)
      {
       edit_stage = edit;

       //  
       init_prog_memory(ee_buff);

       miss_trg = true;

       //         
       if(source == eeprom)
       {
        // 
        i2c_bank_read(bank, ee_buff);

        beep(300);
        goto m_mcycle;
       }
       // flash
       else
       {
        // crc
        u32 crc = calc_pgm_crc((u8*)See[bank].pg_step, flash_prog_size);

        //crc  -    ,  
        if(crc == See[bank].crc)
        {
         //    
         for(int i=0; i < flash_prog_size; i++) ee_buff[i] = See[bank].pg_step[i];

         beep(300);
         goto m_mcycle;
        }
        //crc  ,  
        else
        {
         strcpy(s0,"PROG CRC FAILED ");
         strcpy(s1,"press any key.. ");
        }
       }
      }
     }break;
     //  ,   
     case edit:
     {
      //    
      if(ops == OP_PGMM)
      {
       dev_st = work;
       retval = 1;
      }
      // 
      else if(ops == OP_SR_EE)
      {
       strcpy(s0,"EDIT PROG MODE  ");
       strcpy(s1,"SAVE? YES:1 NO:0");

       edit_stage  = save;
      }
      // 
      else if(ops == OP_RS_EE)
      {
       itr = 0;
       ops = OP_NULL;
       retval = 1;
      }
      // ,  
      else if(ops == OP_FCx && keys[2]==0)
      {
       init_prog_memory(ee_buff);

       pc_reset = 1;
       prg_c(ops,ee_buff,pc_reset);

       prg_visualisation(ee_buff);
      }
      // 
      else
      {
       if(miss_trg == true) miss_trg = false;
       else prog_cnt = prg_c(ops,ee_buff,pc_reset);

       prg_visualisation(ee_buff, prog_cnt);
      }
     }break;

     case save:
     {
      static bool trg = false;

      if(ops == OP_0 && trg == false)
      {
       miss_trg = true;
       edit_stage = edit;
      }
      else
      {
       if(ops == OP_1 || trg == true)
       {
        trg = true;

        if(prog_choice(source, bank, ops, ee_bnum)==true)
        {
         edit_stage = edit;
         miss_trg   = true;
         trg        = false;

         strcpy(s1,"press any key.. ");

         //  
         int length = get_program_length();

         if(length == 0) strcpy(s0,"nothing to save ");
         else
         {
          // eeprom
          if(source == eeprom)
          {
           save_bank(bank, ee_buff);
           beep(300);

           strcpy(s0,"SAVE COMPLETE.. ");
          }
          // flash
          else
          {
           //  
           if(length > flash_prog_size)
           {
            strcpy(s0,"too big to save ");

           }
           else
           {
            write_flash_pgm(ee_buff, bank);
            beep(300);

            strcpy(s0,"SAVE COMPLETE.. ");
           }
          }
         }
        }
       }
      }
     }break;
    }
   }

   if(dev_st!=prog_edit){itr=0;}
  }
  break;
 }

 // true       
 if(retval)
 {
  retval=0;
  return 1;
 }
 return 0;
}