#include <storage_routines.h>

#define DEF_PRECISION 4
#define STORAGE_NUM 20 //   2

u8 ee_buff[0xff];
extern double storage[STORAGE_NUM];

int bank;
enum Storage_source{eeprom, flash} source;


bool read_mregs(void)
{
 bool opres = i2c_bank_read(0, ee_buff);

 for(u8 i=0;i<STORAGE_NUM;i++)storage[i] = *((double*)ee_buff+i);

 return opres;
}

bool save_mregs(void)
{
 bool opres=1;
 double mreg[2];

 for(u8 i=0;i<STORAGE_NUM;i+=2)
 {
  mreg[0] = storage[i];
  mreg[1] = storage[i+1];

  opres &= i2c_buffered_write(i/2*16, (u8*)mreg);
 }

 return opres;
}

bool save_bank(u8 bank_num, u8 ee_buff[0xff])
{
 bool opres=1;
 u16 addr = bank_num*256;

 for(u16 i=0;i<0xff;)
 {
  opres &= i2c_buffered_write(addr+i,(u8*)(ee_buff+i));
  i += 16;
 }
 return opres;
}

u8 check_eeprom_bnum(void)
{
 u8 bank_num=0;
 const char ee_cps[]="num_of_banks:";
 bool i2c_ok=1;

 i2c_ok &= read_mregs();

 if(strcmp((char*)ee_cps,(char*)ee_buff+0xfe-sizeof(ee_cps))==0) bank_num = ee_buff[0xff-1];
 else
 {
  //      prom

  for(u8 i=0;i<4;i++)
  {
   i2c_ok &= i2c_write(i*256+0xfe, i+1);

   u8 tmp;
   i2c_ok &= i2c_read(i*256+0xfe, &tmp);

   if(i+1==tmp) bank_num++;
  }

  if(bank_num)
  {
   for(u8 i=0; i<sizeof(ee_cps);i++)
   {
    i2c_ok &= i2c_write(0xfe-sizeof(ee_cps)+i, ee_cps[i]);
   }

   i2c_ok &= i2c_write(0xfe, bank_num);

   //  
   for(int i=0;i<STORAGE_NUM-1;i++)storage[i]=0;
   save_mregs();

   lock_keys.key = 0;
   save_key(&lock_keys);

   options.precision = 4;
   save_options(&options);
  }
 }

 if(i2c_ok == false)return 0;

 return bank_num;
}

//-----------------------------------------------
bool save_key(ee_keys *keys)
{
 bool opres=1;

 keys->key_0 = keys->key;
 keys->key_1 = ~keys->key_0;

 //    
 opres &= i2c_buffered_write(sizeof storage, (u8*)keys);

 return opres;
}

bool restore_key(ee_keys *keys)
{
 bool opres =1;

 int m0 = sizeof(storage);
 int m1 = sizeof(ee_keys) - sizeof (u64);

 for(int i=m0;i<m0+m1;i++) opres &= i2c_read(i, (u8*)keys+i-m0);

 if(keys->key_0 == ~keys->key_1) keys->key = keys->key_0;
 else keys->key = 0;

 if(!opres) return 0;

 return 1;
}

void restore_key_from_ram(ee_keys *keys)
{
 int m0 = sizeof(storage);
 int m1 = sizeof(ee_keys) - sizeof(u64);

 for(int i=m0;i<m0+m1;i++) *((u8*)keys+i-m0) = ee_buff[i];

 if(keys->key_0 == ~keys->key_1) keys->key = keys->key_0;
 else keys->key = 0;
}

//-----------------------------------------------
bool save_options(const ee_options *options)
{
 int tmp = sizeof storage + sizeof lock_keys - sizeof (u64);
 // 16        
 bool opres = i2c_buffered_write(tmp, (u8*)options);

 return opres;
}

bool restore_options(ee_options *options)
{
 bool opres = 1;
 int m0 = sizeof(storage) + sizeof (ee_keys) - sizeof (u64);
 int m1 = sizeof(ee_options);

 for(int i=m0;i<m0+m1;i++) opres &= i2c_read(i, (u8*)options+i-m0);

 return opres;
}

void restore_options_from_ram(ee_options *options)
{
 int m0 = sizeof(storage) + sizeof (ee_keys) - sizeof(u64);
 int m1 = sizeof(ee_options);

 for(int i=m0;i<m0+m1;i++) *((u8*)options+i-m0) = ee_buff[i];
}

//-----------------------------------------------

//      (eeprom  flash)
bool prog_choice(Storage_source &source, int &bank, opcodes ops, int ee_b=0, bool reset=0)
{
 static int stage;

 if(reset == true) stage = 0;

 m_bgn:

 switch (stage)
 {
  case 0:
  {
   stage++;

   // eeprom        flash
   if(ee_b > 1)
   {
    strcpy(s0,"EEPROM OR FLASH?");
    strcpy(s1,"EEPROM:0 FLASH:1");
   }
   //eeprom ,         flash
   else
   {
    ops = OP_1;
    goto m_bgn;
   }
  }break;
  //  eeprom  flash
  case 1:
  {
   // eeprom
   //      
   if(ops == OP_0)
   {
    stage++;

    source = eeprom;

    //         
    if(ee_b < 2)
    {
     ops = OP_1;
     goto m_bgn;
    }
    else
    {
     strcpy(s0,"EEPROM BANK NUM?");
     strcpy(s1,"(1 2 3)available");
    }
   }
   // flash
   //  
   else if(ops == OP_1)
   {
    stage++;

    source = flash;

    strcpy(s0,"FLASH PROG NUM? ");
    strcpy(s1,"(0...%)available");
   }
  }break;
  //  
  case 2:
  {
   if(source == eeprom)
   {
    if     (ops == OP_1) {bank = 1; stage++; goto m_bgn;}
    else if(ops == OP_2) {bank = 2; stage++; goto m_bgn;}
    else if(ops == OP_3) {bank = 3; stage++; goto m_bgn;}
   }
   //source == flash
   else
   {
    switch(ops)
    {
     case OP_0   : bank = 0 ; stage++; goto m_bgn;
     case OP_COM : bank = 1 ; stage++; goto m_bgn;
     case OP_MNS : bank = 2 ; stage++; goto m_bgn;
     case OP_SEXP: bank = 3 ; stage++; goto m_bgn;
     case OP_Cx  : bank = 4 ; stage++; goto m_bgn;
     case OP_1   : bank = 5 ; stage++; goto m_bgn;
     case OP_2   : bank = 6 ; stage++; goto m_bgn;
     case OP_3   : bank = 7 ; stage++; goto m_bgn;
     case OP_XY  : bank = 8 ; stage++; goto m_bgn;
     case OP_ENT : bank = 9 ; stage++; goto m_bgn;
     case OP_4   : bank = 10; stage++; goto m_bgn;
     case OP_5   : bank = 11; stage++; goto m_bgn;
     case OP_6   : bank = 12; stage++; goto m_bgn;
     case OP_SUM : bank = 13; stage++; goto m_bgn;
     case OP_MUL : bank = 14; stage++; goto m_bgn;
     case OP_7   : bank = 15; stage++; goto m_bgn;
     case OP_8   : bank = 16; stage++; goto m_bgn;
     case OP_9   : bank = 17; stage++; goto m_bgn;
     case OP_SBT : bank = 18; stage++; goto m_bgn;
     case OP_DIV : bank = 19; stage++; goto m_bgn;
    }
   }
  }break;
  //   
  case 3:
  {
   stage = 0;
   return true;

  }break;
 }

 return false;
}
